@@ -8,9 +8,7 @@ Testing focused on pikepdf.Pdf
 from __future__ import annotations
 
 import locale
-import os
 import shutil
-import sys
 import zlib
 from contextlib import nullcontext
 from io import BytesIO, StringIO
@@ -20,6 +18,7 @@ from subprocess import CalledProcessError
 from unittest.mock import Mock
 
 import pytest
+from conftest import fails_if_pypy
 
 import pikepdf
 from pikepdf import Name, PasswordError, Pdf, PdfError, Stream
@@ -31,7 +30,7 @@ from pikepdf._exceptions import DependencyError
 @pytest.fixture
 def trivial(resources):
     with Pdf.open(
-        resources / 'pal-1bit-trivial.pdf', access_mode=pikepdf.AccessMode.mmap
+        resources / 'pal-1bit-trivial.pdf'  # , access_mode=pikepdf.AccessMode.mmap
     ) as pdf:
         yield pdf
 
@@ -162,10 +161,10 @@ class TestStreams:
 
     def test_save_stream(self, trivial, outdir):
         pdf = trivial
-        pdf.save(outdir / 'nostream.pdf', static_id=True)
+        pdf.save(outdir / 'nostream.pdf', deterministic_id=True)
 
         bio = BytesIO()
-        pdf.save(bio, static_id=True)
+        pdf.save(bio, deterministic_id=True)
         bio.seek(0)
 
         with (outdir / 'nostream.pdf').open('rb') as saved_file:
@@ -342,6 +341,7 @@ def test_check(resources):
         assert 'parse error while reading' in problems[0]
 
 
+@fails_if_pypy
 def test_check_specialized_decoder_fallback(resources, monkeypatch):
     class MissingJBIG2Decoder(pikepdf.jbig2.JBIG2DecoderInterface):
         def check_available(self):
